%---------------------------------------------------------
%
%   SPA_ARMA_PIS.M
%
%   Provides an ARMA spectral estimate which is maximum
% entropy satisfying correlation constraint (number of poles)
% and cepstrum constrains (number of ceros). The routine
% requires:
%
%   xin(nsam)   Input signal (real valued) of nsam samples
%   hmod(nres)  Actual spectral dessity with nres the
%               resolution for frequency plot.
%
%   Demands from the control screen
%
%   nq          Order of the denominator (# of poles)
%   np          Order of the numerator (# of ceros)
%
% The routine provides:
%
%   outd(np)    The numerator coefficients outn(1)=1
%   outn(nq)    The denominator coefficients outd(1)=1
%   sigma       The input noise power
%   A frequency plot in Figure(3) depicting the actual spectrum
%   in blue color together with the estimate in red color
%
%   Miguel Angel Lagunas                June 2007
%
%---------------------------------------------------------------
    [sigma,outn,outd]=spa_arma_pis(xinf,hmod);
    disp('ARMA-PISA');
    nres=length(hmod);nsam=length(xinf);
    np=input('Numerator order...... ');
    nq=input('Denominator order.... ');
    % Computes autocorrelation matrix ra(nq,nq).Routine SPA_COR.M
    ra1=spa_corc(xinf,nq);
    % Compute cepstrum
    % Determines the size of the starting correlation matrix, to be used
    % to compute cepstrum as a tradeoff from the maximum size, much
    % greater than the order np and matrix stability.
    mk=min([4 floor(nsam/(np+1))]);
    if mk<1;
        s=['The data record length available is low for the order selected'];
        break;
    else;
        nle=round(nsam/mk);
    end;clear mk;
    % Dispalys at the command window the size of the correlation matrix
    % to be used in the cepstrum estimate
    disp(['Order for cepstral R matrix..' num2str(nle)]);
    ra2=spa_corc(xinf,nle);
    % Eigendecomposition of the correlation matrix for cepstrum
    [u d]=eig(ra2);
    % Pisarenko framework for cepstrum estimation
    d1=diag(log(diag(d)));clear d;
    ca1=u*d1*u';clear ra2 u d1;
    % Compute frequency cepstrum
    for i=1:nsam;
        sc=exp(jj*2*pi*(i-1)*(0:1:(nle-1))'/nsam);
        logc(i)=real(sc'*ca1*sc/(nle));
    end;clear sc ca1;
    % Compute time cepstrum
    auso=real(ifft(logc,nsam));cuso=auso(2:np+1);    
    cep=(1:1:np).*cuso(1:np);clear cuso auso;
    % Compute impulse response
    i=2;h(1)=1;
    while i<np+0.5;
       h(i)=0;
       for k=1:i-1;
          h(i)=h(i)+h(k)*cep(i-k);
       end;
       h(i)=h(i)/(i-1);i=i+1;
     end;
     % Write the system Haenkel matrix
     hh=zeros(np,nq);
     for i=1:np;
       for k=1:nq;
         if k<=i;
            hh(i,k)=h(i-k+1);
         end;
       end;
     end;
     % Find minimum eigenvector
     [u d]=eig(ra1,hh'*hh);
     sas=diag(d);disp(['Eigenvalues']);disp(sas)
     dmin=10e6;
     for i=1:nq;
       if abs(d(i,i))<dmin;
         dmin=abs(d(i,i));nqq=i;
       end;
     end;
     % Denominator coefficients
     outd=real(u(:,nqq)');outd=outd/outd(1);sigma=1/sqrt(d(nqq,nqq));
     hha=hh/sigma;
     % Numerator coefficients
     outn=outd*hha';
     % Normalization
     sigma=sigma*outn(1);outn=outn/outn(1);
     disp('Input noise power');disp(sigma^2);
     disp('Coefficients denominator');disp(outd);
     disp('Numerator coefficients');disp(outn/outd(1));
     % Compute spectral estimate with resolution nres
     aus=20*log10(abs((fft(outn,2*nres)./fft(outd,2*nres))));
     aus=aus+(10*log10(sigma^2))*ones(1,2*nres);
     % Plot estimate together with actual spectrum hmod
     figure(3);clf;
     plot((1:1:nres),aus(1:nres),'r');grid;xlabel('frequency');
     ylabel('dB');title(['MEM-ARMA estimation (red) actual(blue)' stip] );
     hold on;plot((1:1:nres),hmod,'b');hold off;grid;
%---------------------------------------------------------------------
function ra=spa_corc(x,nq);
n=length(x);
l1=1;l2=nq;xf=[];xb=[];luso=0;
while l2<=n;
  auso=x(l1:l2);
  xf=[xf;auso];xb=[xb;fliplr(auso)];
  l1=l1+1;l2=l2+1;luso=luso+1;
end;
ra=0.5*(xf'*xf+xb'*xb)/luso;
%------------------------------------------------------